@@ -10,13 +10,15 @@ Many options in the SDK are unused. For example:
 * No tags or metadata are set on any SDK objects
 * Only defaults are used for ACL/access levels (no public access for created file systems)
 
-Instead of trying to shoehorn functionality like the above into the pyarrow.PyFileSystem API, it is
-recommended to use the SDK separately to use this sort of functionality.
+Instead of trying to shoehorn functionality like the above into the pyarrow.PyFileSystem API,
+it is recommended to use the SDK separately to use this sort of functionality.
 """
 
 import os
 import datetime
 import io
+import typing
+import dataclasses
 
 import azure.core.exceptions
 import azure.storage.filedatalake
@@ -25,8 +27,60 @@ import pyarrow.fs
 
 def _parse_azure_ts(last_modified):
     # Mon, 17 Aug 2020 12:19:35 GMT
-    fmt = "%a, %d %b %Y %H:%M:%S %Z"
-    return datetime.datetime.strptime(last_modified, fmt)
+    if isinstance(last_modified, str):
+        fmt = "%a, %d %b %Y %H:%M:%S %Z"
+        return datetime.datetime.strptime(last_modified, fmt)
+    else:
+        return last_modified
+
+
+@dataclasses.dataclass
+class Timeouts:
+    """Timeouts passed to azure.storage.filedatalake operations
+
+    The value of these are provided as the timeout kwarg to the
+    corresponding object in azure.storage.filedatalake. Timeout
+    units are in seconds."""
+
+    file_client_timeout: typing.Optional[int]
+    file_system_timeout: typing.Optional[int]
+    datalake_service_timeout: typing.Optional[int]
+    directory_client_timeout: typing.Optional[int]
+
+    def __init__(
+        self,
+        file_client_timeout: typing.Optional[int] = None,
+        file_system_timeout: typing.Optional[int] = None,
+        datalake_service_timeout: typing.Optional[int] = None,
+        directory_client_timeout: typing.Optional[int] = None
+    ):
+        """
+        :param file_client_timeout: timeout in seconds to pass to
+            azure.storage.filedatalake.DataLakeFileClient methods
+        :param file_system_timeout: timeout in seconds to pass to
+            azure.storage.filedatalake.FileSystemClient methods
+        :param datalake_service_timeout: timeout in seconds to pass to
+            azure.storage.filedatalake.DataLakeServiceClient methods
+        :param datalake_service_timeout: timeout in seconds to pass to
+            azure.storage.filedatalake.DataLakeDirectoryClient methods
+        """
+
+        self.file_client_timeout = file_client_timeout
+        self.file_system_timeout = file_system_timeout
+        self.datalake_service_timeout = datalake_service_timeout
+        self.directory_client_timeout = directory_client_timeout
+
+
+DEFAULT_TIMEOUTS = Timeouts()
+
+
+def document_timeout(target, timeout_used):
+    def wrap_method(method):
+        method.__doc__ = f"""Return result of azure.storage.filedatalake.{target.__qualname__}
+
+        Affected by self.timeouts.{timeout_used}"""
+        return method
+    return wrap_method
 
 
 class DatalakeGen2File(io.IOBase):
@@ -39,8 +93,9 @@ class DatalakeGen2File(io.IOBase):
     DEFAULT_BLOCK_SIZE = 5 * 2 ** 20
 
     def __init__(self, file_client: azure.storage.filedatalake.DataLakeFileClient,
-                 mode='rb', block_size='default'):
+                 mode='rb', block_size='default', timeouts=DEFAULT_TIMEOUTS):
         super(DatalakeGen2File, self).__init__()
+        self.timeouts = timeouts
         self.file_client = file_client
         self.mode = mode
         self.block_size = self.DEFAULT_BLOCK_SIZE if block_size == 'default' else block_size
@@ -54,6 +109,43 @@ class DatalakeGen2File(io.IOBase):
         if self.mode == 'wb':
             self.file_client.create_file()
 
+    @document_timeout(
+        azure.storage.filedatalake.DataLakeFileClient.get_file_properties,
+        "file_client_timeout")
+    def get_file_properties(self):
+        return self.file_client.get_file_properties(
+            timeout=self.timeouts.file_client_timeout
+        )
+
+    @document_timeout(
+        azure.storage.filedatalake.DataLakeFileClient.append_data,
+        "file_client_timeout")
+    def append_data(self, data, offset, length):
+
+        # timeout is undocumented here, but is still being used in the sdk code
+        return self.file_client.append_data(
+            data, offset, length, timeout=self.timeouts.file_client_timeout
+        )
+
+    @document_timeout(
+        azure.storage.filedatalake.DataLakeFileClient.flush_data,
+        "file_client_timeout")
+    def flush_data(self, offset):
+
+        # timeout is undocumented here, but is still being used in the sdk code
+        return self.file_client.flush_data(
+            offset, timeout=self.timeouts.file_client_timeout
+        )
+
+    @document_timeout(
+        azure.storage.filedatalake.DataLakeFileClient.download_file,
+        "file_client_timeout")
+    def download_file(self, loc, length):
+
+        return self.file_client.download_file(
+            loc, length, timeout=self.timeouts.file_client_timeout
+        )
+
     def tell(self):
         return self.loc
 
@@ -62,7 +154,11 @@ class DatalakeGen2File(io.IOBase):
             raise ValueError("Seek only available in read mode")
         if not 0 <= whence <= 2:
             raise ValueError(f'Invalid whence {whence}, should be 0, 1 or 2')
-        new_loc = [loc, self.loc + loc, self.file_client.get_file_properties().size + loc][whence]
+        new_loc = [
+            loc,
+            self.loc + loc,
+            self.get_file_properties().size + loc
+        ][whence]
         if new_loc < 0:
             raise ValueError("Seek before start of file")
         self.loc = new_loc
@@ -79,6 +175,9 @@ class DatalakeGen2File(io.IOBase):
             self.flush()
         return out
 
+    def writeable(self):
+        return self.mode in {'wb', 'ab'}
+
     def flush(self):
         if self.closed:
             raise ValueError("Flush on closed file")
@@ -87,13 +186,13 @@ class DatalakeGen2File(io.IOBase):
 
         if self.offset is None:
             if self.mode == 'ab':
-                self.offset = self.file_client.get_file_properties().size
+                self.offset = self.get_file_properties().size
             else:
                 self.offset = 0
 
-        self.file_client.append_data(self.buffer.getvalue(), self.offset, self.buffer.tell())
+        self.append_data(self.buffer.getvalue(), self.offset, self.buffer.tell())
         self.offset += self.buffer.tell()
-        self.file_client.flush_data(self.offset)
+        self.flush_data(self.offset)
         self.buffer = io.BytesIO()
 
     def read(self, length=-1):
@@ -102,26 +201,31 @@ class DatalakeGen2File(io.IOBase):
         if self.closed:
             raise ValueError('I/O on closed file')
         if length < 0:
-            length = self.file_client.get_file_properties().size - self.loc
+            length = self.get_file_properties().size - self.loc
         if length == 0:
             return b''
 
-        downloader = self.file_client.download_file(self.loc, length)
-        return downloader.readall()
+        return self.download_file(self.loc, length).readall()
 
 
 class FilesystemHandler(pyarrow.fs.FileSystemHandler):
     """
     Handler for a single file system within an azure storage account.
 
-    Use this if you do not have access to the account itself, f. ex. if you have a SAS token that
-    has access only to a single file system.
+    Use this if you do not have access to the account itself, f. ex. if you have a SAS token
+    that has access only to a single file system.
     """
 
-    def __init__(self, file_system_client: azure.storage.filedatalake.FileSystemClient, prefix_fs=False):
+    def __init__(
+            self,
+            file_system_client: azure.storage.filedatalake.FileSystemClient,
+            prefix_fs=False,
+            timeouts=DEFAULT_TIMEOUTS
+    ):
         """
         :param file_system_client:
         :param prefix_fs: If True, prefix the name of the file system to all generated paths
+        :param timeouts: :class:`Timeouts` for datalake gen2 operations
         :type file_system_client: azure.storage.filedatalake.FileSystemClient
         :type prefix_fs: bool
 
@@ -129,7 +233,8 @@ class FilesystemHandler(pyarrow.fs.FileSystemHandler):
         """
         super().__init__()
         self.prefix_fs = prefix_fs
-        self.file_system_client: azure.storage.filedatalake.FileSystemClient = file_system_client
+        self.file_system_client = file_system_client
+        self.timeouts = timeouts
 
     def _prefix(self, path):
         if self.prefix_fs and path:
@@ -139,15 +244,71 @@ class FilesystemHandler(pyarrow.fs.FileSystemHandler):
         else:
             return path
 
+    @document_timeout(
+        azure.storage.filedatalake.FileSystemClient.get_paths,
+        "file_system_timeout")
+    def get_paths(self, path, recursive=False):
+        return self.file_system_client.get_paths(
+            path, recursive=recursive, timeout=self.timeouts.file_system_timeout
+        )
+
+    @document_timeout(
+        azure.storage.filedatalake.DataLakeFileClient.rename_file,
+        "file_client_timeout")
+    def rename_file(self, src_path, dest_path):
+        return self.file_system_client.get_file_client(src_path).rename_file(
+            dest_path, timeout=self.timeouts.file_client_timeout
+        )
+
+    @document_timeout(
+        azure.storage.filedatalake.DataLakeDirectoryClient.rename_directory,
+        "directory_client_timeout")
+    def rename_directory(self, src_path, dest_path):
+        return self.file_system_client.get_directory_client(src_path).rename_directory(
+            dest_path, timeout=self.timeouts.directory_client_timeout
+        )
+
+    @document_timeout(
+        azure.storage.filedatalake.FileSystemClient.create_directory,
+        "file_system_timeout")
+    def create_directory(self, path):
+        return self.file_system_client.create_directory(
+            path, timeout=self.timeouts.file_system_timeout)
+
+    @document_timeout(
+        azure.storage.filedatalake.FileSystemClient.delete_directory,
+        "file_system_timeout")
+    def delete_directory(self, path):
+        return self.file_system_client.delete_directory(
+            path, timeout=self.timeouts.file_system_timeout
+        )
+
+    @document_timeout(
+        azure.storage.filedatalake.DataLakeFileClient.delete_file,
+        "file_client_timeout")
+    def delete_file_(self, path):
+        return self.file_system_client.get_file_client(path).delete_file(
+            timeout=self.timeouts.file_client_timeout
+        )
+
     @classmethod
-    def from_account_name(cls, account_name, file_system_name, credential=None):
+    def from_account_name(
+            cls,
+            account_name,
+            file_system_name,
+            credential=None,
+            timeouts=DEFAULT_TIMEOUTS
+    ):
         """
         Create from storage account name, file system name and credential
 
         :param account_name:
         :param file_system_name:
-        :param credential: Any valid valid value to pass as credential to azure.storage.filedatalake.FileSystemClient
-        :type credential: str for SAS tokens, None for public access, any credential from azure.identity
+        :param credential: Any valid valid value to pass as credential to
+            azure.storage.filedatalake.FileSystemClient
+        :param timeouts: :class:`Timeouts` for datalake gen2 operations
+        :type credential: str for SAS tokens, None for public access,
+            any credential from azure.identity
         :return: FilesystemHandler
         """
         client = azure.storage.filedatalake.FileSystemClient(
@@ -155,16 +316,20 @@ class FilesystemHandler(pyarrow.fs.FileSystemHandler):
             file_system_name,
             credential=credential
         )
-        return cls(client)
+        return cls(client, timeouts)
 
     def __eq__(self, other):
         if isinstance(other, FilesystemHandler):
-            return self.file_system_client == other.file_system_client
+            return (
+                self.file_system_client == other.file_system_client
+                and self.timeouts == other.timeouts)
         return NotImplemented
 
     def __neq__(self, other):
         if isinstance(other, FilesystemHandler):
-            return self.file_system_client != other.file_system_client
+            return (
+                self.file_system_client != other.file_system_client
+                or self.timeouts != other.timeouts)
         return NotImplemented
 
     def get_type_name(self):
@@ -172,12 +337,19 @@ class FilesystemHandler(pyarrow.fs.FileSystemHandler):
         return f"abfs+{self.file_system_client.account_name}/{self.file_system_client.file_system_name}"
 
     def normalize_path(self, path: str):
-        return path
+        return path.lstrip('/').rstrip('/')
 
-    def _create_file_info(self, path_properties: azure.storage.filedatalake._models.PathProperties):
+    def _create_file_info(
+            self,
+            path_properties: azure.storage.filedatalake._models.PathProperties
+    ):
+        if path_properties.is_directory:
+            path_type = pyarrow.fs.FileType.Directory
+        else:
+            path_type = pyarrow.fs.FileType.File
         return pyarrow.fs.FileInfo(
             self._prefix(path_properties.name),
-            pyarrow.fs.FileType.Directory if path_properties.is_directory else pyarrow.fs.FileType.File,
+            path_type,
             size=path_properties.content_length,
             mtime=_parse_azure_ts(path_properties.last_modified)
         )
@@ -188,7 +360,7 @@ class FilesystemHandler(pyarrow.fs.FileSystemHandler):
             return
         try:
             parent = os.path.dirname(path)
-            path_property_result = self.file_system_client.get_paths(parent, recursive=False)
+            path_property_result = self.get_paths(parent, recursive=False)
             for path_properties in path_property_result:
                 if path_properties.name == path:
                     if not path_properties.is_directory:
@@ -208,7 +380,7 @@ class FilesystemHandler(pyarrow.fs.FileSystemHandler):
                 pyarrow.fs.FileType.Directory
             )
         parent = os.path.dirname(path)
-        listing = self.file_system_client.get_paths(parent, recursive=False)
+        listing = self.get_paths(parent, recursive=False)
         for path_properties in listing:
             if path_properties.name == path:
                 return self._create_file_info(path_properties)
@@ -216,19 +388,22 @@ class FilesystemHandler(pyarrow.fs.FileSystemHandler):
 
     def get_file_info(self, paths: [str]):
         return [
-            self._get_file_info(path) for path in paths
+            self._get_file_info(self.normalize_path(path)) for path in paths
         ]
 
     def get_file_info_selector(self, selector: pyarrow.fs.FileSelector):
         try:
-            self._verify_is_dir(selector.base_dir)
+            self._verify_is_dir(self.normalize_path(selector.base_dir))
         except FileNotFoundError:
             if selector.allow_not_found:
                 return []
             else:
                 raise
 
-        listing = self.file_system_client.get_paths(selector.base_dir, recursive=selector.recursive)
+        listing = self.get_paths(
+            self.normalize_path(selector.base_dir),
+            recursive=selector.recursive
+        )
 
         return [
             self._create_file_info(path_properties)
@@ -236,48 +411,54 @@ class FilesystemHandler(pyarrow.fs.FileSystemHandler):
         ]
 
     def create_dir(self, path, recursive):
+        path = self.normalize_path(path)
         if recursive:
-            self.file_system_client.create_directory(path)
+            self.create_directory(path)
         else:
             parent = os.path.dirname(path)
             self._verify_is_dir(parent)
-            self.file_system_client.create_directory(path)
+            self.create_directory(path)
 
     def delete_dir(self, path):
+        path = self.normalize_path(path)
         self._verify_is_dir(path)
-        self.file_system_client.delete_directory(path)
+        self.delete_directory(path)
 
     def delete_dir_contents(self, path, accept_root_dir=False):
+        path = self.normalize_path(path)
         self._verify_is_dir(path)
         if not accept_root_dir and path in {'', '/'}:
             raise ValueError('Attempt to delete root dir with accept_root_dir=False')
-        for path_properties in self.file_system_client.get_paths(path, recursive=False):
+        for path_properties in self.get_paths(path, recursive=False):
             if path_properties.is_directory:
-                self.file_system_client.delete_directory(path_properties.name)
+                self.delete_directory(path_properties.name)
             else:
-                self.file_system_client.get_file_client(path_properties.name).delete_file()
+                self.delete_file_(path_properties.name)
 
     def delete_root_dir_contents(self, path):
         self.delete_dir_contents(path=None)
 
     def delete_file(self, path):
+        path = self.normalize_path(path)
         file_info: pyarrow.fs.FileInfo = self.get_file_info([path])[0]
         if not file_info.is_file:
             raise IsADirectoryError(self._prefix(path))
-        self.file_system_client.get_file_client(path).delete_file()
+        self.delete_file_(path)
 
     def move(self, src, dest):
         # This is a simple rename. Caveat: the dest path is not relative to the file_system,
         # the azure-sdk expects the file system to be prefixed to the new path.
+        src = self.normalize_path(src)
+        dest = self.normalize_path(dest)
         src_info = self.get_file_info([src])[0]
         if src_info.type == pyarrow.fs.FileType.Directory:
-            dir_client = self.file_system_client.get_directory_client(src)
-            dir_client.rename_directory(new_name=dir_client.file_system_name + '/' + dest)
+            self.rename_directory(src, self.file_system_client.file_system_name + '/' + dest)
         else:
-            file_client = self.file_system_client.get_file_client(src)
-            file_client.rename_file(new_name=file_client.file_system_name + '/' + dest)
+            self.rename_file(src, self.file_system_client.file_system_name + '/' + dest)
 
     def copy_file(self, src, dest):
+        src = self.normalize_path(src)
+        dest = self.normalize_path(dest)
         try:
             info = self.get_file_info([dest])[0]
             if info.type == pyarrow.fs.FileType.Directory:
@@ -291,22 +472,26 @@ class FilesystemHandler(pyarrow.fs.FileSystemHandler):
                 out.write(source.read())
 
     def open_input_stream(self, path):
+        path = self.normalize_path(path)
         self._verify_is_file(path)
         fc = self.file_system_client.get_file_client(path)
-        return pyarrow.PythonFile(DatalakeGen2File(fc, mode='rb'))
+        return pyarrow.PythonFile(DatalakeGen2File(fc, mode='rb', timeouts=self.timeouts))
 
     def open_input_file(self, path):
+        path = self.normalize_path(path)
         self._verify_is_file(path)
         fc = self.file_system_client.get_file_client(path)
-        return pyarrow.PythonFile(DatalakeGen2File(fc, mode='rb'))
+        return pyarrow.PythonFile(DatalakeGen2File(fc, mode='rb', timeouts=self.timeouts))
 
     def open_output_stream(self, path):
+        path = self.normalize_path(path)
         fc = self.file_system_client.get_file_client(path)
-        return pyarrow.PythonFile(DatalakeGen2File(fc, mode='wb'))
+        return pyarrow.PythonFile(DatalakeGen2File(fc, mode='wb', timeouts=self.timeouts))
 
     def open_append_stream(self, path):
+        path = self.normalize_path(path)
         fc = self.file_system_client.get_file_client(path)
-        return pyarrow.PythonFile(DatalakeGen2File(fc, mode='ab'))
+        return pyarrow.PythonFile(DatalakeGen2File(fc, mode='ab', timeouts=self.timeouts))
 
     def _verify_is_file(self, path):
         info = self.get_file_info([path])[0]
@@ -316,15 +501,24 @@ class FilesystemHandler(pyarrow.fs.FileSystemHandler):
     def to_fs(self):
         return pyarrow.fs.PyFileSystem(self)
 
+
 class AccountHandler(pyarrow.fs.FileSystemHandler):
     """Handler for a single azure storage account.
 
     Use this to to access an Azure Storage account with hierarchial namespace enabled.
     """
 
-    def __init__(self, datalake_service: azure.storage.filedatalake.DataLakeServiceClient):
+    def __init__(
+            self,
+            datalake_service: azure.storage.filedatalake.DataLakeServiceClient,
+            timeouts=DEFAULT_TIMEOUTS,
+            fs_handler_cls=FilesystemHandler
+    ):
         """
         :param datalake_service: data lake account service
+        :param timeouts: :class:`Timeouts` for datalake gen2 operations
+        :param fs_handler_cls: How to create FilesystemHandlers for interacting with
+            individual file systems in this account
         :type datalake_service: azure.storage.filedatalake.DataLakeServiceClient
 
         https://azuresdkdocs.blob.core.windows.net/$web/python/azure-storage-file-datalake/12.1.1/azure.storage.filedatalake.html#azure.storage.filedatalake.DataLakeServiceClient
@@ -332,30 +526,47 @@ class AccountHandler(pyarrow.fs.FileSystemHandler):
         super().__init__()
         self.datalake_service = datalake_service
         self.file_system_handlers = {}
+        self.timeouts = timeouts
+        self.fs_handler_cls = fs_handler_cls
 
     @classmethod
-    def from_account_name(cls, account_name, credential=None):
+    def from_account_name(
+            cls,
+            account_name,
+            credential=None,
+            timeouts=DEFAULT_TIMEOUTS,
+            fs_handler_cls=FilesystemHandler
+    ):
         """
         Create from storage account name and credential
 
         :param account_name:
-        :param credential: Any valid valid value to pass as credential to azure.storage.filedatalake.FileSystemClient
-        :type credential: str for SAS tokens, None for public access, any credential from azure.identity
-        :return: FilesystemHandler"""
+        :param credential: Any valid valid value to pass as credential to
+            azure.storage.filedatalake.FileSystemClient
+        :param timeouts: :class:`Timeouts` for datalake gen2 operations
+        :param fs_handler_cls: How to create FilesystemHandlers for interacting with
+            individual file systems in this account
+        :type credential: str for SAS tokens, None for public access, any credential
+            from azure.identity
+        :return: pyarrow.fs.FileSystemHandler"""
         datalake_service = azure.storage.filedatalake.DataLakeServiceClient(
             f'https://{account_name}.dfs.core.windows.net',
             credential
         )
-        return cls(datalake_service)
+        return cls(datalake_service, timeouts)
 
     def __eq__(self, other):
         if isinstance(other, AccountHandler):
-            return self.datalake_service == other.datalake_service
+            return (
+                self.datalake_service == other.datalake_service
+                and self.timeouts == other.timeouts)
         return NotImplemented
 
     def __neq__(self, other):
         if isinstance(other, AccountHandler):
-            return self.datalake_service != other.datalake_service
+            return (
+                self.datalake_service != other.datalake_service
+                or self.timeouts != other.timeouts)
         return NotImplemented
 
     def get_type_name(self):
@@ -363,11 +574,10 @@ class AccountHandler(pyarrow.fs.FileSystemHandler):
         return f'abfs+{self.datalake_service.account_name}'
 
     def normalize_path(self, path):
-        return path
+        return path.lstrip('/').rstrip('/')
 
-    @staticmethod
-    def _split_path(path):
-        path = path.lstrip('/')
+    def _split_path(self, path):
+        path = self.normalize_path(path)
         if '/' not in path:
             return path, ''
         fs_name, *rest = path.split('/')
@@ -380,7 +590,11 @@ class AccountHandler(pyarrow.fs.FileSystemHandler):
         if fs_name in self.file_system_handlers:
             return self.file_system_handlers[fs_name]
         else:
-            new_fs_handler = FilesystemHandler(self.datalake_service.get_file_system_client(fs_name), prefix_fs=True)
+            new_fs_handler = self.fs_handler_cls(
+                self.datalake_service.get_file_system_client(fs_name),
+                prefix_fs=True,
+                timeouts=self.timeouts
+            )
             return self.file_system_handlers.setdefault(fs_name, new_fs_handler)
 
     def _get_file_info(self, path):
@@ -395,15 +609,38 @@ class AccountHandler(pyarrow.fs.FileSystemHandler):
     def get_file_info(self, paths):
         return [self._get_file_info(path) for path in paths]
 
+    @document_timeout(
+        azure.storage.filedatalake.DataLakeServiceClient.list_file_systems,
+        "datalake_service_timeout")
+    def list_file_systems(self):
+        return self.datalake_service.list_file_systems(
+            timeout=self.timeouts.datalake_service_timeout)
+
+    @document_timeout(
+        azure.storage.filedatalake.DataLakeServiceClient.create_file_system,
+        "datalake_service_timeout")
+    def create_file_system(self, name):
+        return self.datalake_service.create_file_system(
+            name, timeout=self.timeouts.datalake_service_timeout
+        )
+
+    @document_timeout(
+        azure.storage.filedatalake.DataLakeServiceClient.delete_file_system,
+        "datalake_service_timeout")
+    def delete_file_system(self, name):
+        return self.datalake_service.delete_file_system(
+            name, timeout=self.timeouts.datalake_service_timeout
+        )
+
     def get_file_info_selector(self, selector: pyarrow.fs.FileSelector):
         fs_name, path = self._split_path(selector.base_dir)
         if not fs_name:
             file_system_data = [
                 pyarrow.fs.FileInfo(fs.name, pyarrow.fs.FileType.Directory, mtime=fs.last_modified)
-                for fs in self.datalake_service.list_file_systems()
+                for fs in self.list_file_systems()
             ]
             if selector.recursive:
-                for fs in self.datalake_service.list_file_systems():
+                for fs in self.list_file_systems():
                     file_system_data.extend(self._fs(fs.name).get_file_info_selector(selector))
             return file_system_data
         else:
@@ -417,19 +654,20 @@ class AccountHandler(pyarrow.fs.FileSystemHandler):
 
         if recursive or not path:
             try:
-                fs_client = self.datalake_service.create_file_system(fs_name)
-                self.file_system_handlers[fs_name] = FilesystemHandler(fs_client, prefix_fs=True)
+                fs_client = self.create_file_system(fs_name)
+                self.file_system_handlers[fs_name] = self.fs_handler_cls(
+                    fs_client, prefix_fs=True, timeouts=self.timeouts)
             except azure.core.exceptions.ResourceExistsError:
                 pass
         if path:
-            if not fs_name in [fs.name for fs in self.datalake_service.list_file_systems()]:
+            if fs_name not in [fs.name for fs in self.list_file_systems()]:
                 raise FileNotFoundError(fs_name)
             self._fs(fs_name).create_dir(path, recursive)
 
     def delete_dir(self, path):
         fs_name, path = self._split_path(path)
         if not path:
-            self.datalake_service.delete_file_system(fs_name)
+            self.delete_file_system(fs_name)
         else:
             self._fs(fs_name).delete_dir(path)
 
@@ -437,8 +675,8 @@ class AccountHandler(pyarrow.fs.FileSystemHandler):
         fs_name, path = self._split_path(path)
         if not fs_name:
             if accept_root_dir:
-                for fs in self.datalake_service.list_file_systems():
-                    self.datalake_service.delete_file_system(fs.name)
+                for fs in self.list_file_systems():
+                    self.delete_file_system(fs.name)
             else:
                 raise ValueError('Attempt to remove root dir with accept_root_dir=False')
         else:
@@ -455,7 +693,7 @@ class AccountHandler(pyarrow.fs.FileSystemHandler):
         elif not path:
             raise IsADirectoryError(fs_name)
         else:
-            if fs_name not in [fs.name for fs in self.datalake_service.list_file_systems()]:
+            if fs_name not in [fs.name for fs in self.list_file_systems()]:
                 raise FileNotFoundError(fs_name)
             self._fs(fs_name).delete_file(path)
 
@@ -480,13 +718,13 @@ class AccountHandler(pyarrow.fs.FileSystemHandler):
                     raise ValueError(f'{dest} is non-empty directory')
             if fi.type != dest_fi.type:
                 raise ValueError(f'src {src} is {fi.type}, but dest {dest} is {dest_fi.type}')
-        except FileNotFoundError as dest_doesnt_exist:
+        except FileNotFoundError:
             pass
 
         if fi.is_file:
-            self._fs(src_fs).file_system_client.get_file_client(src_path).rename_file(dest)
+            self._fs(src_fs).rename_file(src_path, dest)
         else:
-            self._fs(src_fs).file_system_client.get_directory_client(src_path).rename_directory(dest)
+            self._fs(src_fs).rename_directory(src_path, dest)
 
     def copy_file(self, src, dest):
         try:
@@ -526,4 +764,3 @@ class AccountHandler(pyarrow.fs.FileSystemHandler):
 
     def to_fs(self):
         return pyarrow.fs.PyFileSystem(self)
-
